@@ -23,7 +23,7 @@
 
 #include <stdarg.h>
 
-#define OPENCDK_VERSION "0.4.1"
+#define OPENCDK_VERSION "0.4.5"
 
 #ifdef __cplusplus
 extern "C" {
@@ -100,21 +100,26 @@ typedef enum
   CDK_Error_No_Keyring = 21,
   CDK_Wrong_Format = 22,
   CDK_Inv_Packet_Ver = 23,
-  CDK_Too_Short = 24
+  CDK_Too_Short = 24,
+  CDK_Unusable_Key = 25,
 } cdk_error_t;
 
 
 enum
 {
-  CDK_CTLF_SET      =  0,
-  CDK_CTLF_GET      =  1,
-  CDK_CTL_DIGEST    = 10,
-  CDK_CTL_CIPHER    = 11,
-  CDK_CTL_ARMOR     = 12,
-  CDK_CTL_COMPRESS  = 13,
-  CDK_CTL_COMPAT    = 14,
-  CDK_CTL_OVERWRITE = 15,
-  CDK_CTL_S2K       = 16
+  CDK_CTLF_SET          =  0,
+  CDK_CTLF_GET          =  1,
+  CDK_CTL_DIGEST        = 10,
+  CDK_CTL_CIPHER        = 11,
+  CDK_CTL_ARMOR         = 12,
+  CDK_CTL_COMPRESS      = 13,
+  CDK_CTL_COMPAT        = 14,
+  CDK_CTL_OVERWRITE     = 15,
+  CDK_CTL_S2K           = 16,
+  CDK_CTL_KEYCACHE_ON   = 17,
+  CDK_CTL_KEYCACHE_FREE = 18,
+  CDK_CTL_FORCE_DIGEST  = 19,
+  CDK_CTL_TRUSTMODEL    = 20
 };
 
 enum
@@ -149,6 +154,7 @@ enum
   CDK_MD_RMD160  = 3,
   CDK_MD_MD2     = 5,
   CDK_MD_TIGER   = 6,
+  CDK_MD_SHA256  = 8,
 };
 
 enum
@@ -174,6 +180,14 @@ enum
   CDK_PREFTYPE_ZIP  = 3
 };
 
+enum
+{
+  CDK_REVCOD_NOREASON   = 0x00,
+  CDK_REVCOD_SUPERCEDED = 0x01,
+  CDK_REVCOD_COMPROMISED= 0x02,
+  CDK_REVCOD_NOLONGUSED = 0x03
+};
+
 enum
 {
   CDK_ARMOR_MESSAGE   = 0,
@@ -205,6 +219,18 @@ enum
 #define CDK_DBTYPE_ARMORED 100
 #define CDK_DBTYPE_KEYRING 100
 
+
+enum
+{
+  CDK_CRYPTYPE_NONE    = 0,
+  CDK_CRYPTYPE_ENCRYPT = 1,
+  CDK_CRYPTYPE_DECRYPT = 2,
+  CDK_CRYPTYPE_SIGN    = 3,
+  CDK_CRYPTYPE_VERIFY  = 4,
+  CDK_CRYPTYPE_EXPORT  = 5,
+  CDK_CRYPTYPE_IMPORT  = 6
+};
+
 enum
 {
   CDK_KEY_VALID   = 0,
@@ -354,7 +380,6 @@ typedef struct
   unsigned is_primary:1;
   unsigned has_expired:1;
   unsigned is_revoked:1;
-  int req_usage;
 } cdkPKT_secret_key;
 
 
@@ -478,6 +503,7 @@ typedef struct cdk_packet_s CDK_PACKET;
 
 
 /*-- main.c --*/
+/* memory routines */
 typedef void (*CDK_LOG_FNC) (void *, int, const char *, va_list);
 void cdk_set_log_level (int lvl);
 void cdk_set_log_handler (CDK_LOG_FNC logfnc, void * opaque);
@@ -494,7 +520,7 @@ void * cdk_realloc (void * ptr, size_t size);
 void * cdk_salloc (size_t size, int clear);
 char * cdk_strdup (const char * ptr);
 void cdk_free (void * ptr);
-
+/* session handle routines */
 int cdk_handle_new (CDK_HD * r_ctx);
 void cdk_handle_free (CDK_HD c);
 void cdk_handle_set_keydb (CDK_HD hd, CDK_KEYDB_HD db);
@@ -503,6 +529,17 @@ int cdk_handle_control( CDK_HD hd, int action, int cmd, ... );
 void cdk_handle_set_callback (CDK_HD hd,
                               void (*cb) (void *opa, int type, const char * s),
                               void * cb_value);
+void cdk_handle_set_passphrase_cb( CDK_HD hd,
+                                   char *(*cb) (void *opa, const char *prompt),
+                                   void * cb_value );
+
+/* shortcuts for some controls */
+#define cdk_handle_set_armor( a, val ) \
+  cdk_handle_control( (a), CDK_CTLF_SET, CDK_CTL_ARMOR, (val) )
+
+#define cdk_handle_set_compress( a, algo, level ) \
+  cdk_handle_control( (a), CDK_CTLF_SET, CDK_CTL_COMPRESS, (algo), (level) )
+
 
 /*-- cipher.c --*/
 void cdk_set_progress_handler (void (*cb)(void * hd, unsigned off,
@@ -557,16 +594,11 @@ unsigned long cdk_pk_fingerprint_get_keyid (const unsigned char * fpr,
 unsigned long cdk_pk_get_keyid (cdkPKT_public_key * pk, unsigned long * keyid);
 unsigned long cdk_sk_get_keyid (cdkPKT_secret_key * sk, unsigned long * keyid);
 unsigned long cdk_sig_get_keyid (cdkPKT_signature * sg, unsigned long * keyid);
-typedef char * (*CDK_PASSPHRASE_CB) (const char *);
 int cdk_sk_unprotect (cdkPKT_secret_key * sk, const char * pw);
 int cdk_sk_protect (cdkPKT_secret_key * sk, const char * pw);
-void cdk_passphrase_cb_set (CDK_PASSPHRASE_CB pass_fnc);
+int cdk_pk_from_secret_key( cdkPKT_secret_key * sk,
+                            cdkPKT_public_key ** ret_pk );
 
-/*-- verify.c --*/
-int cdk_file_verify (CDK_HD hd, const char * file, const char * output);
-unsigned long cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what);
-const void * cdk_sig_get_data_attr (CDK_HD hd, int idx, int what);
-     
 /*-- seskey.c --*/
 int cdk_sesskey_new( CDK_SESSKEY * r_sk );
 void cdk_sesskey_free( CDK_SESSKEY sk );
@@ -578,9 +610,10 @@ int cdk_dek_from_passphrase( CDK_DEK * ret_dek, int cipher_algo,
                              CDK_S2K_HD s2k, int mode, const char * pw );
 int cdk_dek_encode_pkcs1( CDK_DEK dek, int nbits, CDK_SESSKEY * r_esk );
 int cdk_dek_decode_pkcs1( CDK_DEK * ret_dek, CDK_SESSKEY esk );
-int cdk_dek_extract( CDK_DEK * ret_dek, cdkPKT_pubkey_enc * enc,
+int cdk_dek_extract( CDK_DEK * ret_dek, CDK_HD hd, cdkPKT_pubkey_enc * enc,
                      cdkPKT_secret_key * sk );
 void cdk_dek_set_mdc_flag( CDK_DEK dek, int val );
+/* string to key */
 int cdk_s2k_new (CDK_S2K_HD * ret_s2k, int mode, int algo,
                  const unsigned char * salt);
 void cdk_s2k_free (CDK_S2K_HD s2k);
@@ -618,6 +651,8 @@ int cdk_stream_set_compress_flag (CDK_STREAM s, int algo, int level);
 int cdk_stream_set_hash_flag (CDK_STREAM s, int algo);
 int cdk_stream_set_text_flag (CDK_STREAM s, const char * lf);
 int cdk_stream_kick_off (CDK_STREAM inp, CDK_STREAM out);
+int cdk_stream_mmap( CDK_STREAM s, unsigned char ** ret_buf,
+                     size_t * ret_count );
 
 /*-- keydb.c --*/
 int cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, void * data, size_t count);
@@ -625,21 +660,22 @@ int cdk_keydb_open (CDK_KEYDB_HD hd, CDK_STREAM * ret_kr);
 int cdk_keydb_check_sk( CDK_KEYDB_HD hd, unsigned long * keyid );
 int cdk_keydb_search_new (CDK_DBSEARCH * r_dbs, int type, void * desc);
 void cdk_keydb_search_free (CDK_DBSEARCH dbs);
-int cdk_keydb_search (CDK_KEYDB_HD hd, CDK_DBSEARCH ks, CDK_KBNODE * r_key);
+int cdk_keydb_search (CDK_KEYDB_HD hd, CDK_DBSEARCH ks, CDK_KBNODE * ret_key);
 int cdk_keydb_search_auto (CDK_KEYDB_HD hd, const char *desc,
-                           CDK_KBNODE * r_knode);
+                           CDK_KBNODE * ret_key);
 void cdk_keydb_free (CDK_KEYDB_HD hd);
 int cdk_keydb_get_bykeyid (CDK_KEYDB_HD hd, unsigned long * keyid,
-                           CDK_KBNODE * r_key);
+                           CDK_KBNODE * ret_key);
 int cdk_keydb_get_byfpr (CDK_KEYDB_HD hd, const unsigned char * fpr,
-                         CDK_KBNODE * r_key);
+                         CDK_KBNODE * ret_key);
 int cdk_keydb_get_bypattern (CDK_KEYDB_HD hd, const char * patt,
-                             CDK_KBNODE * r_pkt);
+                             CDK_KBNODE * ret_key);
 int cdk_keydb_get_pk (CDK_KEYDB_HD khd, unsigned long * keyid,
                       cdkPKT_public_key ** ret_pk);
 int cdk_keydb_get_sk (CDK_KEYDB_HD khd, unsigned long * keyid,
                       cdkPKT_secret_key ** ret_sk);
 int cdk_keydb_get_keyblock (CDK_STREAM inp, CDK_KBNODE * ret_key);
+int cdk_keydb_idx_rebuild( CDK_KEYDB_HD hd );
 int cdk_keydb_export (CDK_KEYDB_HD hd, CDK_STREAM out, CDK_STRLIST remusr);
 int cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode, int result[4]);
 
@@ -649,7 +685,7 @@ int cdk_kbnode_read_from_mem (CDK_KBNODE * ret_node,
                              const unsigned char * buf, size_t buflen);
 int cdk_kbnode_write_to_mem (CDK_KBNODE node,
                              unsigned char * buf, size_t * r_nbytes);
-void cdk_kbnode_release (CDK_KBNODE n);
+void cdk_kbnode_release (CDK_KBNODE node);
 CDK_KBNODE cdk_kbnode_walk (CDK_KBNODE root, CDK_KBNODE * ctx, int all);
 CDK_PACKET * cdk_kbnode_find_packet (CDK_KBNODE node, int pkttype);
 CDK_PACKET * cdk_kbnode_get_packet (CDK_KBNODE node);
@@ -669,12 +705,13 @@ int cdk_pklist_build (CDK_KEYDB_HD hd, CDK_STRLIST remusr,
 void cdk_pklist_release (CDK_KEYLIST pkl);
 int cdk_pklist_encrypt (CDK_KEYLIST pkl, CDK_DEK dek, CDK_STREAM out);
 /* secret key list */
-int cdk_sklist_build (CDK_KEYDB_HD hd, CDK_STRLIST locusr,
-                      CDK_KEYLIST * ret_skl, int unlock, unsigned int use);
+int cdk_sklist_build( CDK_KEYDB_HD db, CDK_HD hd, CDK_STRLIST locusr,
+                      CDK_KEYLIST * ret_skl, int unlock, unsigned int use );
 void cdk_sklist_release (CDK_KEYLIST skl);
 int cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, void * mdctx,
                       int sigclass, int sigver);
-int cdk_sklist_write_onepass (CDK_KEYLIST skl, CDK_STREAM outp, int sigclass);
+int cdk_sklist_write_onepass( CDK_KEYLIST skl, CDK_STREAM outp,
+                              int sigclass, int mdalgo );
 
 /*-- encrypt.c --*/
 int cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
@@ -682,12 +719,27 @@ int cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
 int cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
                       const char * file, const char * output);
 int cdk_file_decrypt (CDK_HD hd, const char * file, const char * output);
+int cdk_data_transform( CDK_HD hd, int mode,
+                        CDK_STRLIST locusr, CDK_STRLIST remusr,
+                        const void * inbuf, size_t insize,
+                        unsigned char ** outbuf, size_t * outsize,
+                        int modval );
 
 /*-- sign.c --*/
+int cdk_stream_sign( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out,
+                     CDK_STRLIST locusr, CDK_STRLIST remusr,
+                     int encryptflag, int sigmode );
 int cdk_file_sign (CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
                    const char * file, const char * output,
                    int sigmode, int encryptflag);
 
+/*-- verify.c --*/
+int cdk_stream_verify( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out );
+int cdk_file_verify (CDK_HD hd, const char * file, const char * output);
+unsigned long cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what);
+const void * cdk_sig_get_data_attr (CDK_HD hd, int idx, int what);
+unsigned char * cdk_sig_get_info_as_xml( CDK_HD hd, int advanced );
+
 /*-- trustdb.c --*/
 int cdk_trustdb_get_validity (CDK_STREAM inp, cdkPKT_user_id * id, int *r_val);
 int cdk_trustdb_get_ownertrust (CDK_STREAM inp, cdkPKT_public_key * pk,
@@ -698,7 +750,6 @@ void cdk_strlist_free (CDK_STRLIST sl);
 CDK_STRLIST cdk_strlist_add (CDK_STRLIST * list, const char * string);
 const char * cdk_strlist_walk (CDK_STRLIST root, CDK_STRLIST * context);
 const char * cdk_check_version (const char * req_version);
-
 /* UTF8 */
 char * cdk_utf8_encode (const char * string);
 char * cdk_utf8_decode (const char * string, size_t length, int delim);
@@ -706,7 +757,7 @@ char * cdk_utf8_decode (const char * string, size_t length, int delim);
 /*-- keyserver.c --*/
 int cdk_keyserver_recv_key (const char * host, int port,
                             const unsigned char * keyid, int kid_type,
-                            CDK_KBNODE * r_knode);
+                            CDK_KBNODE * r_key);
 
 /*-- keygen.c --*/
 int cdk_keygen_new (CDK_KEYGEN_CTX * r_hd);
