@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "gnu" -*- */
 /* minpg.c - Minimal Privacy Guard (MinPG)
  *        Copyright (C) 2002, 2003 Timo Schulz
  *
@@ -24,14 +25,13 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-#ifdef HAVE_LIBPOPT
 #include <stdio.h>
 #include <popt.h>
-#include <opencdk.h>
 #include <string.h>
 #include <time.h>
 #include <stdlib.h>
 
+#include "opencdk.h"
 
 enum
 {
@@ -46,7 +46,9 @@ enum
   cGenKey = 'w',
   cSigList = 'l',
   cEncr = 'e',
+  cEncrFiles = '@',
   cDecr = 'd',
+  cDecrFiles = '$',
   cExport = 'x',
   cImport = 'i',
   cShowpref = 'q',
@@ -62,7 +64,7 @@ enum
   oArmor = 'p',
   oCompat = 'g',
   oNoCompress = '0',
-  oDetachSign = 'j',
+  oDetachSign = 'j',  
 };
 
 
@@ -81,6 +83,7 @@ static struct
   {"MD5", CDK_MD_MD5},
   {"SHA1", CDK_MD_SHA1},
   {"RIPEMD160", CDK_MD_RMD160},
+  {"SHA256", CDK_MD_SHA256},
   {NULL},
 };
 
@@ -142,7 +145,11 @@ struct poptOption arguments[] =
   {"recipient", oRecipient, POPT_ARG_STRING, &user, oRecipient,
    "Use this recipient", 0},
   {"encrypt", cEncr, POPT_ARG_STRING, &file, cEncr, "Encrypt a file", 0},
+  {"encrypt-files", cEncrFiles, POPT_ARG_STRING, &file, cEncrFiles,
+   "Encrypt |files|", 0},
   {"decrypt", cDecr, POPT_ARG_STRING, &file, cDecr, "Decrypt a file", 0},
+  {"decrypt-files", cDecrFiles, POPT_ARG_STRING, &file, cDecrFiles,
+   "Decrypt |files|", 0},
   {"verbose", oVerbose, POPT_ARG_NONE, 0, oVerbose, "Verbosity", 0},
   {"armor", oArmor, POPT_ARG_NONE, 0, oArmor, "Armor output", 0},
   {NULL}
@@ -219,6 +226,7 @@ key_get_letter (int pubkey_algo)
     {
     case CDK_PK_RSA:   return 'R';
     case CDK_PK_RSA_E: return 'r';
+    case CDK_PK_RSA_S: return 'r';
     case CDK_PK_ELG_E: return 'g';
     case CDK_PK_ELG:   return 'G';
     case CDK_PK_DSA:   return 'D';
@@ -290,6 +298,7 @@ get_keyvalid_char (cdkPKT_public_key * pk)
 {
   static char flags[2];
   memset (flags, 0, 2);
+  /*printf ("[invalid=%d]\n", pk->is_invalid);*/
   if (pk->has_expired)
     strcat (flags, "e");
   else
@@ -298,7 +307,9 @@ get_keyvalid_char (cdkPKT_public_key * pk)
     strcat (flags, "r");
   else
     strcat (flags, " ");
-  if (!pk->has_expired && !pk->is_revoked)
+  if (pk->is_invalid)
+    strcat (flags, "?");
+  else if (!pk->has_expired && !pk->is_revoked)
     strcat (flags, "!");
   else
     strcat (flags, " ");
@@ -413,6 +424,8 @@ do_list_keys (CDK_KEYDB_HD hd, CDK_STRLIST keys)
       return 0;
     }
   /* Show all keys */
+  cdk_set_log_level (CDK_LOG_DEBUG);
+  
   rc = cdk_keydb_open (hd, &inp);
   if (rc)
     {
@@ -430,7 +443,6 @@ do_list_keys (CDK_KEYDB_HD hd, CDK_STRLIST keys)
       if (rc)
         break;
     }
-  cdk_stream_close (inp);
   return 0;
 }
 
@@ -479,7 +491,7 @@ do_import_keys (CDK_KEYDB_HD hd, const char * keyfile)
       return -1;
     }
   rc = cdk_stream_open (keyfile, &s);
-  if (!s)
+  if (rc)
     {
       fprintf (stderr, "open `%s' failed: %s\n", keyfile, cdk_strerror (rc));
       return -1;
@@ -549,7 +561,7 @@ do_list_sigs (CDK_KEYDB_HD hd, CDK_STRLIST keys)
 
 
 char *
-get_passphrase (const char *prompt)
+get_passphrase (void * hd, const char *prompt)
 {
   char * p = NULL;
   int maxlen = 255;
@@ -704,6 +716,7 @@ genkey (void)
   printf ("(1) DSA (sign only)\n");
   printf ("(2) DSA and ElGamal\n");
   printf ("(3) DSA and RSA\n");
+  printf ("(4) RSA and ElGamal\n");
   printf ("your choice: ");
   c = fgetc (stdin);
   switch (c)
@@ -711,6 +724,7 @@ genkey (void)
     case '1':
     case '2': 
     case '3': algo = CDK_PK_DSA; break;
+    case '4': algo = CDK_PK_RSA; break;
     default: printf ("invalid public key algorithm\n"); break;
     }
   if (!algo)
@@ -718,10 +732,11 @@ genkey (void)
   cdk_salloc (1, 0);
   cdk_keygen_new (&hd);
   cdk_keygen_set_algo_info (hd, 0, algo, 768);
-  if (c > '1' && c < '4')
+  if (c > '1' && c < '5')
     {
       switch (c)
         {
+        case '4':
         case '2': cdk_keygen_set_algo_info (hd, 1, CDK_PK_ELG_E, 768); break;
         case '3': cdk_keygen_set_algo_info (hd, 1, CDK_PK_RSA_E, 768); break;
         }
@@ -810,7 +825,7 @@ main (int argc, char ** argv)
   char * output = NULL, * s;
   char u[128];
   int c = 0, verbose = 0, i = 0, sigmode = 0, zip_level = 6;
-  int rc = 0;
+  int rc = 0, nfiles = 0;
 
   s = "/home/twoaday/.gnupg/pubring.cdk";
   /*s = "pubring.gpg";*/
@@ -829,12 +844,13 @@ main (int argc, char ** argv)
       return 0;
     }
   
-  /*cdk_passphrase_cb_set (get_passphrase);*/
 
   cdk_handle_new (&hd);
+  cdk_handle_set_passphrase_cb (hd, get_passphrase, NULL);
   cdk_handle_set_callback (hd, callback, NULL);
   cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_OVERWRITE, 1);
   cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_COMPRESS, 2, 5);
+  cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_KEYCACHE_ON, 1);
   /*cdk_set_progress_handler (progr_callback, NULL);*/
   opt_ctx = poptGetContext ("pred", argc, argv, arguments, 0);
   while ((c = poptGetNextOpt (opt_ctx)) >= 0)
@@ -854,7 +870,7 @@ main (int argc, char ** argv)
         case 'p':
           if (verbose)
             fprintf (stdout, "protect output with ascii armor\n");
-          cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_ARMOR, 1);
+          cdk_handle_set_armor( hd, 1 );
           break;
 
 	case 'o':
@@ -870,12 +886,12 @@ main (int argc, char ** argv)
           break;
 
 	case 'h':
-	  fprintf (stderr, "opencdk (MinPG) %s\n\n", VERSION);
+	  fprintf (stderr, "opencdk (MinPG) %s\n\n", OPENCDK_VERSION);
 	  poptPrintHelp (opt_ctx, stdout, 0);
 	  return 0;
 
 	case 'V':
-	  fprintf (stderr, "Minimal Privacy Guard (minpg) %s\n\n", VERSION);
+	  fprintf (stderr, "Minimal Privacy Guard (minpg) %s\n\n", OPENCDK_VERSION);
 	  fprintf (stderr, "Supported algorithms:\n");
 	  fprintf (stderr, "Cipher, Digest: ");
 	  for (i = 0; algo_table[i].name; i++)
@@ -970,6 +986,8 @@ main (int argc, char ** argv)
             {
               cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_DIGEST,
                                   digest_algid);
+              if (!cipher_algid)
+                cipher_algid = CDK_CIPHER_3DES;
               cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_S2K,
                                   3, digest_algid, cipher_algid);
             }
@@ -1002,6 +1020,7 @@ main (int argc, char ** argv)
 	  break;
 
 	case 'e':
+        case '@':
 	  if (!remusr)
 	    {
 	      fprintf (stdout, "No recipient was chosen.\n\n");
@@ -1009,7 +1028,7 @@ main (int argc, char ** argv)
 	      fgets (u, sizeof u, stdin);
 	      cdk_strlist_add (&remusr, user);
 	    }
-          cdk_set_log_level (CDK_LOG_DEBUG);
+          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
 	  output = do_add_ext (file, ".gpg");
 	  if (verbose)
 	    fprintf (stdout, "encrypt `%s' -> %s\n", file, output);
@@ -1017,9 +1036,12 @@ main (int argc, char ** argv)
 	  rc = cdk_file_encrypt (hd, remusr, file, output);
 	  if (rc)
             file_error ("encrypt file", file, rc);
+          if (c == '@')
+            nfiles = 1;
 	  break;
 
 	case 'd':
+        case '$':
           cdk_set_log_level (CDK_LOG_DEBUG);
 	  output = do_remove_ext (file);
 	  if (verbose)
@@ -1031,6 +1053,8 @@ main (int argc, char ** argv)
             file_error ("decrypt file", file, rc);
           else
             print_verify_stats (hd, 0);
+          if (c == '$')
+            nfiles = 2;
 	  break;
 
         case 'w':
@@ -1040,6 +1064,11 @@ main (int argc, char ** argv)
           break;
           
         case 's':
+          cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_FORCE_DIGEST, 1);
+          if (digest_algid)
+            cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_DIGEST,digest_algid);
+          if (verbose)
+            fprintf (stdout, "force digest algo\n");
           if (!remusr)
             {
               fprintf (stdout, "No user was chosen.\n");
@@ -1086,6 +1115,35 @@ main (int argc, char ** argv)
 	}
     }
 
+  if (nfiles)
+    {
+      const char ** files = poptGetArgs (opt_ctx);
+      /*cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_KEYCACHE_FREE, 1);*/
+      while (*files)
+        {
+          if (nfiles == 1)
+            {
+              output = do_add_ext (*files, ".gpg");
+              if (verbose)
+                printf ("encrypt `%s' -> %s\n", *files, output);
+              rc = cdk_file_encrypt (hd, remusr, *files, output);
+            }
+          else if (nfiles == 2)
+            {
+              output = do_remove_ext (*files);
+              if (verbose)
+                printf ("decrypt `%s' -> %s\n", *files, output);
+              rc = cdk_file_decrypt (hd, *files, output);
+            }
+          if (rc)
+              printf ("%s file `%s': %s\n", nfiles==1? "encrypt" : "decrypt",
+                      *files, cdk_strerror (rc));
+          files++;
+          cdk_free (output);
+          output = NULL;
+        }
+    }
+
   cdk_keydb_free (pub_db);
   cdk_keydb_free (sec_db);
   cdk_handle_free (hd);
@@ -1093,11 +1151,3 @@ main (int argc, char ** argv)
   cdk_free (output);
   return 0;
 }
-#else
-int
-main (int argc, char ** argv)
-{
-  printf ("MinPG example is not available.\n");
-  return 0;
-}
-#endif /* HAVE_LIBPOPT */
